import utils;
import hashlib;
import os;
import urlparse;
import subprocess;
import sys;

from buildutils import file_md5_hash, prepend_path, platform_is_win32, platform_is_unix, call_log_file
from package import Config
from Path import Path

class Options:
  def __init__ (self):
    self.no_deps = False
    self.download_only = False
    self.packages = []

  def parse (self, argv):
    for i in argv:
      if i == '--no-deps':
        self.no_deps = True
      elif i == '--download-only':
        self.download_only = True
      else:
        options.packages += [ i ]


options = Options()
options.parse (sys.argv[1:])

if options.no_deps:
  print "(dependencies will not be built)"

config = Config()

#######
# need generic environment builder
# for now, set things manually
if platform_is_win32():
  prepend_path (config.PREFIX.join (Path ("mingw/bin")))
  prepend_path (config.PREFIX.join (Path ("msys/bin")))

prepend_path (config.PREFIX.join (Path ("bin")))

def perform_operation (package, operation, function, hash):
  stamp_file_name = config.BEASTBUILD.join (Path ("stamps/" + package.full_name() + "_" +
                                                  hash + "_" + operation)).native()
  try:
    open (stamp_file_name, "r")
    skip = True
  except IOError:
    skip = False
  if skip:
    print "***** " + operation + " complete already."
  else:
    print "***** " + operation + ":"
    log_file_name = config.BEASTBUILD.join (Path ("logs/" + package.full_name() + "_" + operation + ".log")).unix()
    call_log_file (log_file_name)
    function()
    sys.stdout.flush()
    done_file = open (stamp_file_name, "w")
    done_file.write (operation + " stamp")
    done_file.close()

pkg2file = dict()
shortpkg2pkg = dict()
pkg2package = dict()

if platform_is_win32():
  DIR_SEP = '\\'
else:
  DIR_SEP = '/'

def import_all (path):
  if os.path.isdir (path):
    if not path.endswith (DIR_SEP + "attic"):
      for entry in os.listdir (path):
        import_all (os.path.join (path, entry))
  elif os.path.isfile (path):
    if path.endswith (".py") and not path.endswith ("__.py"):
      name = path[:-3].replace (DIR_SEP, ".")
      __import__ (name)
      pkg_module = sys.modules[name]
      try:
	full_name = pkg_module.package().full_name()
      except:
	utils.die ("%s needs to implement a package() method" % path)
      pkg2file[full_name] = path
      pkg2package[full_name] = pkg_module.package()
      short_name = os.path.basename (path[:-3])
      shortpkg2pkg[short_name] = full_name

if platform_is_win32():
  essential_pkgs = [ 'binutils', 'gcccore', 'gccgxx', 'mingwrtdev', 'mingwrtdll', 'w32apidev',
		     'msyscore', 'msyscrypt', 'cvs', 'vim', 'ssh', 'wget', 'slashx', 'ccache',
		     'nsis', 'nsis_untgz' ]
else:
  essential_pkgs = [ 'slashx' ]

import_all ("packages")

def dependency_order (package_list, complete=None):
  if complete is None:
    complete = set()
  build_list = []
  for package_full_name in package_list:
    package = pkg2package [package_full_name]
    for i in package.depends():
      try:
	ip = shortpkg2pkg [i]
      except:
	utils.die ("can not find package %s which is a dependancy of %s" % (i, package_full_name))
      build_list += dependency_order ([ip], complete)
    if package_full_name not in complete:
      build_list += [package_full_name]
      complete.add (package_full_name)
  return build_list

build_pkgs = map (lambda short_name: shortpkg2pkg [short_name], essential_pkgs + options.packages)
if not options.no_deps:
  build_pkgs = dependency_order (build_pkgs)
build_pkgs = map (lambda full_name: pkg2package [full_name], build_pkgs)

def source_hash (full_name):
  os.chdir (config.BEASTBUILD.native())
  md5 = hashlib.md5()
  # hash all packages .py files that p depends on (including p)
  hash_pkgs = []
  # include essential packages in the list
  for pkg in essential_pkgs:
    hash_pkgs += [shortpkg2pkg [pkg]]
  hash_pkgs += [full_name]
  hash_pkgs = dependency_order (hash_pkgs)
  dep_files = 0
  for package in hash_pkgs:
    md5.update (package)
    md5.update (file_md5_hash (pkg2file[package]))
    dep_files += 1
  # hash all .py files that are part of the build system
  for i in [ 'Path.py', 'beastbuild.py', 'buildutils.py', 'config.py', 'package.py', 'utils.py']:
    md5.update (file_md5_hash (i))
    dep_files += 1
  print "***** hash-depends on %d python files" % dep_files
  return md5.hexdigest()

for package in build_pkgs:
  print "***** " + package.full_name() 
  hash = source_hash (package.full_name())
  perform_operation (package, "download", package.download, hash)
  if not options.download_only:
    perform_operation (package, "extract", package.extract, hash)
    perform_operation (package, "patch", package.patch, hash)
    perform_operation (package, "configure", package.configure, hash)
    perform_operation (package, "compile", package.compile, hash)
    perform_operation (package, "dist_image", package.dist_image, hash)
    perform_operation (package, "install", package.install, hash)
  package.setup_env()

# vim:set sw=2 sts=2:
