import utils
import urlparse
import os
import hashlib
import sys
import subprocess
import stat
from config import config
from Path import Path

def download_and_check (url, md5_hash):
  scheme, host, path, params, qu, fr = urlparse.urlparse (url)
  filename = os.path.basename (path)
  download_dir = config.DOWNLOAD.native()
  if (not os.path.exists (download_dir)):
    os.makedirs (download_dir)
  filename_full = os.path.join (download_dir, filename)
  if (os.path.exists (filename_full)):
    if (file_md5_hash (filename_full) == md5_hash):
      return
  print "downloading " + filename_full + " from " + url
  sys.stdout.flush()
  utils.getFile (url, download_dir)
  local_hash = file_md5_hash (filename_full) 
  if (local_hash == md5_hash):
    print "md5 hash check ok."
    return
  else:
    utils.die ("md5 hash of file " + filename_full + " does not match expected value\n"
             + "expected md5 hash: " + md5_hash + "\n" + "   local md5 hash: " + local_hash)

def git_checkout (repo, project_git, branch):
  if project_git.endswith (".git"):
    project = project_git[:-4]
  else:
    project = project_git
  bb_file_name = config.SRC_DIR.join (Path (project + "/.BEASTBUILD")).native()
  if not os.path.exists (bb_file_name + "_CLONE_OK"):
    os.chdir (config.SRC_DIR.native())
    if repo.startswith ("http://"):
      cmd = "git clone %s/%s" % (repo, project_git)
    else:
      cmd = "git clone %s/%s" % (config.REPOSITORY[repo], project_git)
    print cmd
    status = call_log (["sh", "-c", cmd ])
    if (status != 0):
      utils.die ("cannot clone git repository %s project %s" % (repo, project_git))
    # create stamp file
    file = open (bb_file_name + "_CLONE_OK", "w")
    file.write ("stamp file needed by beastbuild")
    file.close()
  if not os.path.exists (bb_file_name + "_CHECKOUT_OK") and (branch != "master"):
    os.chdir (os.path.join (config.SRC_DIR.native(), project))
    cmd = "git checkout -b %s origin/%s" % (branch, branch)
    print cmd
    status = call_log (["sh", "-c", cmd ])
    if (status != 0):
      utils.die ("cannot checkout branch %s (project %s)" % (branch, project))
    # create stamp file
    file = open (bb_file_name + "_CHECKOUT_OK", "w")
    file.write ("stamp file needed by beastbuild")
    file.close()
  os.chdir (os.path.join (config.SRC_DIR.native(), project))
  cmd = "git pull"
  print cmd
  status = call_log (["sh", "-c", cmd ])
  if (status != 0):
    utils.die ("cannot pull from project %s" % project)
 
def file_md5_hash (path):
  m = hashlib.md5()
  file = open (path, "rb")
  block = file.read (65536)
  while block != "":
    m.update (block)
    block = file.read (65536)
  file.close()
  return m.hexdigest()

def configure (cmd, dir):
  if (dir != "."):
    os.chdir (os.path.join (config.SRC_DIR.native(), dir))
  cmd_string = ""
  for i in cmd:
    cmd_string += i + " ";
  print "configure: " + cmd_string
  sys.stdout.flush()
  status = call_log (cmd)
  if (status != 0):
    utils.die ("error (%d) while configuring package" % status)

def make (cmd, dir):
  if (dir != "."):
    os.chdir (os.path.join (config.SRC_DIR.native(), dir))
  cmd_string = ""
  for i in cmd:
    cmd_string += i + " ";
  print "make: " + cmd_string
  sys.stdout.flush()
  status = call_log (cmd)
  if (status != 0):
    utils.die ("error (%d) while compiling package" % status)

### patch file functions
def apply_patch(patch, dir, patch_depth):
  if (dir != "."):
    os.chdir (os.path.join (config.SRC_DIR.native(), dir))
  cmd = "set -o pipefail ; stripm < %s | patch -p%d" % (config.BEASTBUILD.join (Path (patch)).unix(), patch_depth)
  print "apply_patch: " + cmd
  sys.stdout.flush()
  status = call_log (["sh", "-c", cmd])
  if (status != 0):
    utils.die ("error: can not apply patch " + patch)

### rm -rf
# shutil.rmtree (dir) fails with write protected files on windows
def rm_rf (path):
  try:
    mode = os.lstat(path).st_mode
    if stat.S_ISDIR(mode):
      for entry in os.listdir (path):
	rm_rf (os.path.join (path, entry))
      os.rmdir (path)
    else:
      try:
        os.remove (path)
      except:
        os.chmod (path, 0777)
        os.remove (path)
  except OSError,e:
    if e.errno != os.errno.ENOENT:  # ignore non-existant files
      utils.die ("error: can not delete '" + path + "' from cwd=" + os.getcwd() + ":" + os.strerror (e.errno))

#--------------------------------------------------------------------------------------------
# subprocess with logfile output
#--------------------------------------------------------------------------------------------

from subprocess import Popen, PIPE
import thread, threading
import os, sys

thread_lock = threading.Lock()
log_file = False

def out_reader (file):
  while True:
    line = file.readline ()
    if line == '':
      return
    thread_lock.acquire()
    sys.stdout.write (line)
    log_file.write (line)
    log_file.flush()
    thread_lock.release()


def call_log_file (file_name):
  global log_file
  try:
    log_file = open (file_name, 'w')
  except:
    utils.die ("error: can't write log file: '" + file_name + "'")

def call_log (cmd):
  proc = Popen(cmd, bufsize=0, stdin=PIPE, stdout=PIPE, stderr=PIPE)
  t1 = thread.start_new_thread (out_reader, (proc.stdout,))
  t2 = thread.start_new_thread (out_reader, (proc.stderr,))
  return proc.wait()

## platform specific code
#
def platform_is_win32():
  return sys.platform == "win32"

def platform_is_unix():
  return not platform_is_win32()

def prepend_path (path):
  if platform_is_win32():
    old_path = os.environ['Path']
    new_path = path.win32() + ";" + old_path
    os.environ['Path'] = new_path
  else:
    old_path = os.environ['PATH']
    new_path = path.unix() + ":" + old_path
    os.environ['PATH'] = new_path

# vim:set sw=2 sts=2:
