from buildutils import download_and_check, configure, make, apply_patch, rm_rf
import utils
from package import Package
from Path import Path

class MkTemp(Package):
  def full_name (self):
    return "mktemp-1.6"

  def download (self):
    download_and_check ("ftp://ftp.mktemp.org/pub/mktemp/mktemp-1.6.tar.gz",
                        "3e66f91f8a39c7dc0a67b158aeb9c2ac")

  def extract (self):
    self.clear_and_unpack ("mktemp-1.6.tar.gz", "mktemp-1.6")

  def patch (self):
    apply_patch ("mktemp-1.6.diff", "mktemp-1.6", 1)
    apply_patch ("mktemp-1.6-2.diff", "mktemp-1.6", 1)

  def configure (self):
    cflags = "-g -Wall -O2" if self.config.ENABLE_DEBUG else "-Wall -O2"
    #cflags += " -I" + self.config.PREFIX.join (Path ("include")).msys()
    configure ([ "sh", "-c", "CFLAGS='" + cflags + "' ./configure " +
                 "--prefix " + self.config.PREFIX.msys()], "mktemp-1.6")

  def compile (self):
    self.compile_with_make ("mktemp-1.6")

  def dist_image (self):
    img_path = self.config.TMP.join (Path (self.full_name()))
    rm_rf (img_path.win32())
    bin_path = img_path.join (Path ("bin"))
    utils.makeDirs (bin_path)
    make ([ "cp", "mktemp.exe", bin_path.msys() ], "mktemp-1.6")
    self.tar_dist_image (img_path)
    rm_rf (img_path.win32())

  def install (self):
    self.install_with_dist_image()

def package():
  return MkTemp()
