#include <stdio.h>
#ifdef WIN32
#include <io.h>	/* win32 */
#endif
#include <fcntl.h>

bool
is_text (int ch)
{
    return (ch >= 0x20 || ch == '\t');
}

int
main (int argc, char **argv)
{
    int ch;
    int lf = 0, cr_lf = 0;

#ifdef WIN32
    setmode(fileno(stdin), O_BINARY);
#endif

    enum State { TXT, SEEN_CR, SEEN_LF, SEEN_CR_LF, BAD } state = TXT;

    while ((ch = getchar()) >= 0)
    {
	if (state == TXT)
	{
	    if (is_text (ch))
		state = TXT;
	    else if (ch == '\r')
		state = SEEN_CR;
	    else if (ch == '\n')
		state = SEEN_LF;
	    else
		state = BAD;
	}
	else if (state == SEEN_CR)
	{
	    if (ch == '\n')
		state = SEEN_CR_LF;
	    else
		state = BAD;
	}
	else if (state == SEEN_LF)
	{
	    if (is_text (ch))
	    {
		lf++;
		state = TXT;
	    }
	    else if (ch == '\r')
	    {
		lf++;
		state = SEEN_CR;
	    }
	    else if (ch == '\n')
	    {
		lf++;
		state = SEEN_LF;
	    }
	    else
	    {
		state = BAD;
	    }
	}
	else if (state == SEEN_CR_LF)
	{
	    if (is_text (ch))
	    {
		cr_lf++;
		state = TXT;
	    }
	    else if (ch == '\r')
	    {
		cr_lf++;
		state = SEEN_CR;
	    }
	    else if (ch == '\n')
	    {
		cr_lf++;
		state = SEEN_LF;
	    }
	    else
	    {
		state = BAD;
	    }
	}
    }
    if (state == BAD)
    {
	printf ("BAD\n");
    }
    else
    {
	if (lf && !cr_lf)
	    printf ("LF\n");
	if (cr_lf && !lf)
	    printf ("CRLF\n");
	if (lf && cr_lf)
	    printf ("LF-CRLF\n");
	if (!lf && !cr_lf)
	    printf ("NONE\n");
    }
    return 0;
}

// vim:set ts=8 sts=4 sw=4:
