###########################################################
# taken from kde windows port emerge
###########################################################

# this file contains some helper functions for emerge

# copyright:
# Holger Schroeder <holger [AT] holgis [DOT] net>
# Patrick Spendrin <ps_ml [AT] gmx [DOT] de>


import httplib
import ftplib
import os
import sys
import re
import urlparse
import shutil
import zipfile
import tarfile
import hashlib
import subprocess
import __builtin__
import imp
from Path import Path
from config import config
from buildutils import platform_is_win32, platform_is_unix

### fetch functions

#FIXME: get this from somewhere else:
if platform_is_win32():
  WGetExecutable = os.path.join( config.PREFIX.native(), "bin", "wget.exe" )
else:
  WGetExecutable = os.path.join( config.PREFIX.native(), "bin", "wget" )


def getFiles( urls, destdir ):
    debug( "getfiles called. urls: %s" % urls, 1 )
    # make sure distfiles dir exists
    if ( not os.path.exists( destdir ) ):
        os.makedirs( destdir )

    for url in urls.split():
        #print "getfiles url:", url
        if ( not getFile( url, destdir ) ):
            return False

    return True

def getFile( url, destdir ):
    debug( "getFile called. url: %s" % url, 1 )
    if url == "":
        error( "fetch: no url given" )
        return False


    wgetpath = WGetExecutable             ### FIXME
    if ( os.path.exists( wgetpath ) ):
        return wgetFile( url, destdir )

    scheme, host, path, params, qu, fr = urlparse.urlparse( url )


    filename = os.path.basename( path )
    debug( "%s\n%s\n%s\n%s" % ( scheme, host, path, filename ) )

    if ( scheme == "http" ):
        return getHttpFile( host, path, destdir, filename )
    elif ( scheme == "ftp" ):
        return getFtpFile( host, path, destdir, filename )
    else:
        error( "getFile: protocol not understood" )
        return False

def wgetFile( url, destdir ):
    compath = WGetExecutable
    command = "%s -c -t 1 -P %s %s" % ( compath, destdir, url )
    debug( "wgetfile called", 1 )
    attempts = 1
    if url.lower().startswith( "http://downloads.sourceforge.net" ):
        debug( "Detected downloads.sourceforge.net... Trying three times." )
        attempts=3

    while( attempts > 0 ):
        attempts -= 1
        ret = os.system( command )
        debug( "wget ret: %s" % ret )
        if ret == True:
            # success stop early.
            break;

    return ret

def getFtpFile( host, path, destdir, filename ):
    # FIXME check return values here (implement useful error handling)...
    debug( "FIXME getFtpFile called. %s %s" % ( host, path ), 1 )

    outfile = open( os.path.join( destdir, filename ), "wb" )
    ftp = ftplib.FTP( host )
    ftp.login( "anonymous", "johndoe@nowhere.org" )
    ftp.retrbinary( "RETR " + path, outfile.write )

    outfile.close()
    return True

def getHttpFile( host, path, destdir, filename ):
    # FIXME check return values here (implement useful error handling)...
    debug( "getHttpFile called. %s %s" % ( host, path ) , 1 )

    conn = httplib.HTTPConnection( host )
    conn.request( "GET", path )
    r1 = conn.getresponse()
    debug( "status: %s; reason: %s" % ( str( r1.status ), str( r1.reason ) ) )
        
    count = 0
    while r1.status == 302:
        if count > 10:
            print "Redirect loop"
            return False
        count += 1
        scheme, host, path, params, qu, fr = urlparse.urlparse( r1.getheader( "Location" ) )
        debug( "Redirection: %s %s" % ( host, path ), 1 )
        conn = httplib.HTTPConnection( host )
        conn.request( "GET", path )
        r1 = conn.getresponse()
        debug( "status: %s; reason: %s" % ( str( r1.status ), str( r1.reason ) ) )
    
        
    data = r1.read()

    f = open( os.path.join( destdir, filename ), "wb" )
    f.write( data )
    f.close()
    return True


### unpack functions

def unpackFiles( downloaddir, filenames, workdir ):
    cleanDirectory( workdir )

    for filename in filenames:
        debug( "unpacking this file: %s" % filename, 1 )
        if ( not unpackFile( downloaddir, filename, workdir ) ):
            return False

    return True

def unpackFile( downloaddir, filename, workdir ):
    ( shortname, ext ) = os.path.splitext( filename )
    if ( ext == ".zip" ):
        return unZip (os.path.join (downloaddir.native(), filename), workdir)
    elif ( ext == ".tgz" ):
	return unTar (downloaddir.join (Path (filename)), workdir)
    elif ( ext == ".gz" or ext == ".bz2" ):
        ( myshortname, myext ) = os.path.splitext( shortname )
        if ( myext == ".tar" ):
            return unTar (downloaddir.join (Path (filename)), workdir)
        else:
            error( "unpacking %s" % myext )
            return False
    elif ( ext == ".exe" ):
        warning( "unpack ignoring exe file" )
        return True
    error( "dont know how to unpack this file: %s" % filename )
    return False

def unTar( file, destdir ):
    debug( "unTar called. file: %s, destdir: %s" % ( file, destdir ), 1 )
    ( shortname, ext ) = os.path.splitext( file.unix() )

    mode = "r"
    if (ext == ".gz" or ext == ".tgz"):
        mode = "r:gz"
    elif (ext == ".bz2"):
        mode = "r:bz2"

    tar = tarfile.open (file.native(), mode)

    # FIXME how to handle errors here ?
    for foo in tar:
        tar.extract (foo, destdir.native())

    return True

def unZip (file, destdir):
    debug ("unZip called: file %s to destination %s" % (file, destdir.native()), 1)

    if not os.path.exists (destdir.native()):
        os.makedirs (destdir.native())

    zip = zipfile.ZipFile (file)

    for i, name in enumerate( zip.namelist() ):
        if not name.endswith( '/' ):
            dirname = os.path.join (destdir.native(), os.path.dirname (name))

            if not os.path.exists (dirname):
                os.makedirs (dirname)

            outfile = open (os.path.join (destdir.native(), name), 'wb')
            outfile.write (zip.read (name))
            outfile.flush()
            outfile.close()

    return True

### debug functions
def info( message ):
    if verbose() > 0:
        print "beastbuild info: %s" % message
    return True

def debug( message, level=0 ):
    if verbose() > level:
        print "beastbuild debug:", message
    return True

def warning( message ):
    if verbose() > 0:
        print "beastbuild warning: %s" % message
    return True

def debug_line( level=0 ):
    if verbose() > level:
        print "_" * 80

def error( message ):
    if verbose() > 0:
        print >> sys.stderr, "beastbuild error: %s" % message
    return False

def die( message ):
    print >> sys.stderr, "beastbuild fatal error: %s" % message
    exit( 1 )

def verbose():
    verb = os.getenv( "BEASTBUILD_VERBOSE" )
    if ( not verb == None and verb.isdigit() and int(verb) > 0 ):
        return int( verb )
    else:
        return 0

### create directory
def makeDirs (path):
    if os.path.exists (path.native()):
        if os.path.isdir (path.native()):
            return
    else:
        try:
            os.makedirs (path.native())
            if os.path.exists (path.native()) and os.path.isdir (path.native()):
                return
        except:
            pass
    die ("Can't create directory '" + path.native() + "'")
